// App A, pp A46 - A47

#include <fstream.h>
#include <string.h>

const int MAX_NAME_LENGTH = 40;
typedef char nameType[MAX_NAME_LENGTH+1];

struct person
{  nameType  Name;
   double    Salary;
};  // end struct

enum boolean {FALSE, TRUE};

void SearchFileSequentially(nameType FileName, 
                            nameType DesiredName,
									 person& DesiredPerson,
									 boolean& Found)
// --------------------------------------------------------
// Searches a text file sequentially for a desired person.
// Precondition: FileName is the name of a text file of
// names and data about people. Each person is represented
// by two lines in the file: The first line contains the
// person's name, and the second line contains the person's
// salary. DesiredName is the name of the person sought.
// Postcondition: If DesiredName was found in the file, 
// DesiredPerson is a structure that contains the person's 
// name and data, and Found is TRUE. Otherwise, Found is
// FALSE and DesiredPerson is unchanged. The file is
// unchanged and closed.
// --------------------------------------------------------
{
   ifstream InFile(FileName);
   nameType NextName;
   double   NextSalary;

   Found = FALSE;
   while ( (Found == FALSE) && 
           (InFile.getline(NextName, MAX_NAME_LENGTH)) )
   {  InFile >> NextSalary >> ws;  // skip trailing eoln

      if (strcmp(NextName, DesiredName) == 0)
			Found = TRUE;
   }  // end while

   if (Found)
   {  strcpy(DesiredPerson.Name, NextName);
      DesiredPerson.Salary = NextSalary;
   }  // end if

   InFile.close();
}  // end SearchFileSequentially

//=======================FOR DEMONSTRATION ONLY.===========================

void Display(person Person)
// Displays the contents of a person structure.
{  cout << "The structure contains data for:\n"
	<< Person.Name << " whose salary is "
	<< Person.Salary << "\n";
}  // end Display

main()
{
	nameType AName;
	person   DesiredPerson;
	boolean  Found;

	cout << "Enter a name (Q to quit): ";
   cin.getline(AName, MAX_NAME_LENGTH);

   cout << "I read " << AName << ".\n"; //debug

   while (strcmp(AName, "Q") != 0)
   {  SearchFileSequentially("Ch0Names.dat", AName, DesiredPerson, Found);
      if (Found == TRUE)
			Display(DesiredPerson);
      else cout << "No record for " << AName << "\n";

      cout << "Enter another name (Q to quit): ";
      cin.getline(AName, MAX_NAME_LENGTH);

      cout << "I read " << AName << ".\n"; //debug

   }  // end while

	cout << "Program terminated at user request.\n";

   return 0;
}

