// Chap 1, pp 30 -31
#include <iostream.h>
const int LOW_END = 10;    // low end of incomes considered
const int HIGH_END = 100;  // high end of incomes considered
const int TABLE_SIZE = HIGH_END - LOW_END + 1;

typedef int tableType[TABLE_SIZE];
enum boolean {FALSE, TRUE};

int Index(int Group)
// Returns array index that corresponds to Group number.
{
   return Group-LOW_END;
}  // end Index

void ReadData(tableType IncomeData, int Low, int High,
              boolean& DataError)
// ---------------------------------------------------------
// Reads and organizes income statistics.
// Precondition: The calling program gives directions and
// prompts user. Each input line contains exactly two
// integers in the form G N, where N is the number of
// people with an income in the G-thousand-dollar group and
// Low <= G <= High. An input line with zero values
// for both G and N terminates input.
// Postcondition: IncomeData[G-Low] = total number of people
// with an income in the G-thousand-dollar group. If either
// G or N is erroneous (G and N are not both 0, and either
// G < Low, G > High, or N < 0),
// the function ignores the data line, displays a message,
// sets DataError to TRUE, and continues. In this case, the
// calling program should take action. DataError is FALSE
// if the data is free of error.
//----------------------------------------------------------
{
   int Group, Number;  // input values

   DataError = FALSE;  // no data error found as yet

   for (Group = Low; Group <= High; ++Group)  // clear array
      IncomeData[Index(Group)] = 0;

   for (cin >> Group >> Number; (Group != 0) || (Number != 0);
	cin >> Group >> Number)
   {  // Invariant: Group and Number are not both 0
      cout << "Input line specifies that income group "
	   << Group << "\ncontains " << Number
	   << " people.\n";

      if ((Group >= Low) && (Group <= High)
			 && (Number >= 0))
	 //input data is valid -- add it to tally
	 IncomeData[Index(Group)] += Number;

      else  // error in input data
      {  cout << "Error in input data: ";
	 if ( (Group < Low) || (Group > High) )
	 {  cout << "Income group out of range "
		 << Low << " to " << High <<".\n";
	 }

	 if (Number < 0)
	    cout << "Number of people is negative!\n";

	 cout << "This input line will be ignored.\n\n";
	 DataError = TRUE;  // set error flag
      }  // end else
   }  // end while
}  // end ReadData

main()
{
   tableType T;
   boolean Error;

   ReadData (T, LOW_END, HIGH_END, Error);
   for (int I = 0; I < TABLE_SIZE; ++I)
     cout << T[I] << " ";
   cout << endl;
   cout << "Error flag is " << Error << endl;
   return 0;
}

